/*
 * Fortanix SDKMS REST API
 *
 * This is a set of REST APIs for accessing the Fortanix Self-Defending Key Management System. This includes APIs for managing accounts, and for performing cryptographic and key management operations. 
 *
 * API version: 1.0.0-20181211
 * Contact: support@fortanix.com
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package sdkms



type KeyObject struct {
	// Name of the security object.
	Name string `json:"name"`

	// Description of the security object.
	Description string `json:"description,omitempty"`

	// For objects which are not elliptic curves, this is the size in bits (not bytes) of the object. This field is not returned for elliptic curves. 
	KeySize int32 `json:"key_size,omitempty"`

	EllipticCurve *EllipticCurve `json:"elliptic_curve,omitempty"`

	// Account ID of the account this security object belongs to.
	AcctId string `json:"acct_id"`

	// Group ID of the security group that this security object belongs to.
	GroupId string `json:"group_id,omitempty"`

	Creator *CreatorType `json:"creator"`

	// Key ID uniquely identifying this security object.
	Kid string `json:"kid,omitempty"`

	ObjType *ObjectType `json:"obj_type"`

	// Array of key operations enabled for this security object. 
	KeyOps []KeyOperations `json:"key_ops,omitempty"`

	// User-defined metadata for this key. Stored as key-value pairs.
	CustomMetadata map[string]string `json:"custom_metadata,omitempty"`

	Origin *ObjectOrigin `json:"origin"`

	// This field is returned only for asymmetric keys. It contains the public key.
	PubKey string `json:"pub_key,omitempty"`

	// This field is returned only for opaque and secret objects. It contains the contents of the object.
	Value string `json:"value,omitempty"`

	// Whether this security object has cryptographic operations enabled.
	Enabled bool `json:"enabled"`

	// When this security object was created.
	CreatedAt string `json:"created_at"`

	// When this security object was last used.
	LastusedAt string `json:"lastused_at"`

	// Transient key blob.
	TransientKey string `json:"transient_key,omitempty"`

	// True if this key's operations have never contained EXPORT.
	NeverExportable bool `json:"never_exportable"`
}


